@@ -166,7 +166,6 @@ XIV(int, pingTimeout,                           3)
 XIV(int, mailCheckDelay,                        30)
 XIV(int, taskBarCPUSamples,                     20)
 XIV(int, taskBarApmGraphWidth,                  10)
-XIV(int, taskBarGraphHeight,                    20)
 XIV(int, taskBarMEMSamples,                     20)
 XIV(int, focusRequestFlashTime,                 0)
 XIV(int, focusRequestFlashInterval,             250)
@@ -422,7 +421,6 @@ cfoption icewm_preferences[] = {
     OIV("TaskBarWidthPercentage",               &taskBarWidthPercentage, 0, 100, "Task bar width as percentage of the screen width"),
     OSV("TaskBarJustify",                       &taskBarJustify, "Taskbar justify left, right or center"),
     OIV("TaskBarApmGraphWidth",                 &taskBarApmGraphWidth, 1, 1000,  "Width of battery Monitor"),
-    OIV("TaskBarGraphHeight",                   &taskBarGraphHeight, 16, 1000,  "Height of taskbar monitoring applets"),
 
     OIV("XineramaPrimaryScreen",                &xineramaPrimaryScreen, 0, 63, "Primary screen for xinerama where taskbar is shown"),
     OIV("FocusRequestFlashTime",                &focusRequestFlashTime, 0, (3600 * 24), "Number of seconds the taskbar app will blink when requesting focus (0 = forever)"),
@@ -466,6 +464,7 @@ cfoption icewm_preferences[] = {
     OKV("MouseWinSize",                         gMouseWinSize,                  "Mouse binding for window resize"),
     OKV("MouseWinRaise",                        gMouseWinRaise,                 "Mouse binding to raise window"),
     OKV("MouseWinLower",                        gMouseWinLower,                 "Mouse binding to lower window"),
+    OKV("MouseWinTabbing",                      gMouseWinTabbing,               "Mouse binding to create tabs"),
     OKV("KeyWinRaise",                          gKeyWinRaise,                   "Raises the window which currently has input focus."),
     OKV("KeyWinOccupyAll",                      gKeyWinOccupyAll,               "Makes the active window occupy all work spaces."),
     OKV("KeyWinLower",                          gKeyWinLower,                   "Lowers the window which currently has input focus."),
@@ -492,6 +491,15 @@ cfoption icewm_preferences[] = {
     OKV("KeyWinArrangeW",                       gKeyWinArrangeW,                "Moves the active window to the middle left of the screen."),
     OKV("KeyWinArrangeNW",                      gKeyWinArrangeNW,               "Moves the active window to the top left corner of the screen."),
     OKV("KeyWinArrangeC",                       gKeyWinArrangeC,                "Moves the active window to the top middle of the screen."),
+    OKV("KeyWinTileLeft",                       gKeyWinTileLeft,                "Let the active window occupy the left half of the screen"),
+    OKV("KeyWinTileRight",                      gKeyWinTileRight,               "Let the active window occupy the right half of the screen"),
+    OKV("KeyWinTileTop",                        gKeyWinTileTop,                 "Let the active window occupy the top half of the screen"),
+    OKV("KeyWinTileBottom",                     gKeyWinTileBottom,              "Let the active window occupy the bottom half of the screen"),
+    OKV("KeyWinTileTopLeft",                    gKeyWinTileTopLeft,             "Let the active window occupy the top left quarter of the screen"),
+    OKV("KeyWinTileTopRight",                   gKeyWinTileTopRight,            "Let the active window occupy the top right quarter of the screen"),
+    OKV("KeyWinTileBottomLeft",                 gKeyWinTileBottomLeft,          "Let the active window occupy the bottom left quarter of the screen"),
+    OKV("KeyWinTileBottomRight",                gKeyWinTileBottomRight,         "Let the active window occupy the bottom right quarter of the screen"),
+    OKV("KeyWinTileCenter",                     gKeyWinTileCenter,              "Let the active window occupy the center quarter of the screen"),
     OKV("KeyWinSmartPlace",                     gKeyWinSmartPlace,              "Smart place the active window."),
     OKV("KeySysSwitchNext",                     gKeySysSwitchNext,              "Opens the QuickSwitch popup and/or moves the selector in the QuickSwitch popup"),
     OKV("KeySysSwitchLast",                     gKeySysSwitchLast,              "Works like KeySysSwitchNext but moving in the opposite direction."),
@@ -507,6 +515,7 @@ cfoption icewm_preferences[] = {
     OKV("KeySysMenu",                           gKeySysMenu,                    "Activates the IceWM root menu in the lower left corner."),
     OKV("KeySysWindowList",                     gKeySysWindowList,              "Opens the IceWM system window list in the center of the screen."),
     OKV("KeySysWinListMenu",                    gKeySysWinListMenu,             "Shows the window list menu."),
+    OKV("KeySysKeyboardNext",                   gKeySysKeyboardNext,            "Switch to the next keyboard layout in the KeyboardLayouts list."),
     OKV("KeySysAddressBar",                     gKeySysAddressBar,              "Opens the address bar in the task bar where a command can be typed."),
     OKV("KeySysWorkspacePrev",                  gKeySysWorkspacePrev,           "Goes one workspace to the left."),
     OKV("KeySysWorkspaceNext",                  gKeySysWorkspaceNext,           "Goes one workspace to the right."),
